#!/bin/bash

######################################################################
#       FakeBTS.com
#       November 2018
#       v 0.0.1 (LTE 4G, UMTS 3G & GSM 2G active analysis)
#######################################################################
#
#   Copyright (C) 2014 Pedro Cabrera
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    Contact Info:
#    @PCabreraCamara
#    pedro.cabrera@fakebts.com
#
#######################################################################

# BIN PATHS
DIAGPBIN="/opt/diag-parser/diag_parser"
TSHARKBIN="/usr/local/bin/tshark"

# OUTPUT FILES
SCRIPTDIR="$( cd "$( /usr/bin/dirname "$0" )" && pwd )"
OUTDIR=$( echo "${SCRIPTDIR}/outputs")
EVENTDIR=$( echo "${SCRIPTDIR}/events")
TMPDIR=$( echo "${SCRIPTDIR}/tmp")
APPLOG=$( echo "${EVENTDIR}/app.log")
DEBUGLOG=$( echo "${OUTDIR}/cells.log")

# QUECTEL DEVICE CDC, WWAN NAME:
CDCDEV="/dev/cdc-wdm0"
QUECTELDEV="wwan0"

# INITIAL MODE (defaults):
#######################################################################
#	By default Quectel modem LTE mode is going to be used
#	using SIMs provider APN, check the bellow information,
#	it's very important in order to connect:
APNSTRING="apn=orangeworld,username=orange,password=orange,ip-type=4"
#	If you are planning to use "qmi-network", please be sure you
#	have defined your APN config file:
#	https://www.freedesktop.org/software/libqmi/man/latest/qmi-network.1.html
#
#
#	Using UMTS or GSM mode will not force an initial attach to
#	packet network, you can change the preferred network
#	by modifying this variable:
PREFERRED="FALSE"
#######################################################################

# GPS VARIABLES:
#######################################################################
# 	You can add fixed GPS coordinates by using "variable" mode in 
#	command line: -g variable
#	
#	Uncomment the following line in this case:
#	coordinates="36.114647,-115.172813"
#
# 	In order to use built-in GPS using the quectel device (EC20)
#	use the quectel mode in command line: -g quectel
#	
#	If you don't setup any -g option, default 0.0 0.0 coordinates
#	are going to be add in the results: cells and alarms.
GPSMODE="FALSE"

# CELLVERBOSE MODE:
CVERBOSE="FALSE"

function disclaimer {
        echo "CellAnalysis  Copyright (C) 2018 Pedro Cabrera"
        echo "This program comes with ABSOLUTELY NO WARRANTY; for details visit http://www.gnu.org/licenses/gpl.txt"
        echo "This is free software, and you are welcome to redistribute it"
        echo "under certain conditions; for details visit http://www.gnu.org/licenses/gpl.txt."
        echo
}

function usage {
	echo "Usage: $0 -m [MODE] {options: -g -p}"
	echo "   -h			This help."
	echo
	echo "   -m [MODE]		Mandatory, must be: {quectel}.(expected two more modes: xgoldmon and scat)"
	echo
	echo "   -g			Use a GPS device to add coordinates (requires to configure variables in code: GPS Parameters). Could be one of: {variable or quectel}."
	echo
	echo "   -p			Establish Quectel preferred network mode. You have to replug your device every time the preferred network changes. Could be one of: { gsm, umts or lte}."
	echo
}

# CHECK AND CREATE FOLDERS:
function check_folders {

	if [ ! -d "${OUTDIR}" ]; then
	  mkdir -p ${OUTDIR}
	fi

	if [ ! -d "${EVENTDIR}" ]; then
	  mkdir -p ${EVENTDIR}
	fi

	if [ ! -d "${TMPDIR}" ]; then
	  mkdir -p "${TMPDIR}"
	fi
	
}

function check_bin {

	check_folders

	if [ ! -f "${TSHARKBIN}" ]
	then
	        echo "$(date '+%d/%m %H:%M:%S')  Exit !!, ${TSHARKBIN} not found. Please install \"wireshark\" from source code: https://1.eu.dl.wireshark.org/src/wireshark-2.6.5.tar.xz" | tee -a "${APPLOG}"
	        exit 0
	fi
	
	if [ ! -f "${DIAGPBIN}" ]
	then
	        echo "$(date '+%d/%m %H:%M:%S')  Exit !!, ${DIAGPBIN} not found. Please install \"diag-parser\" from source code: https://github.com/moiji-mobile/diag-parser" | tee -a "${APPLOG}"
	        exit 0
	fi
	
	which qmicli > /dev/null
	if [ $? -eq 1 ] 
	then
			echo "$(date '+%d/%m %H:%M:%S')  Exit !!, qmicli not found. Please install \"libqmi-utils\" package." | tee -a "${APPLOG}"
	        exit 0
	fi
	
	which gawk > /dev/null
	if [ $? -eq 1 ] 
	then
			echo "$(date '+%d/%m %H:%M:%S')  Exit !!, gawk not found. Please install \"gawk\" package." | tee -a "${APPLOG}"
	        exit 0
	fi

	echo "$(date '+%d/%m %H:%M:%S')  SW dependencies OK." | tee -a "${APPLOG}"
}

function check_hw {
	
	if [ ! -e ${CDCDEV} ]
	then
		echo "$(date '+%d/%m %H:%M:%S')  Warning, CDC device ${CDCDEV} not found !" | tee -a "${APPLOG}"
		exit 0
	fi
	
	ifconfig ${QUECTELDEV} > /tmp/checkif 2>&1
	grep "Device not found" /tmp/checkif > /dev/null
	if [ $(echo $?) -eq 0 ] 
	then
		echo "$(date '+%d/%m %H:%M:%S')  Warning, WWAN interface ${QUECTELDEV} not found !" | tee -a "${APPLOG}"
		exit 0
	fi
	rm /tmp/checkif
	
	allclear=0
	list=$(dmesg | grep -i "Qualcomm USB modem converter now attached to" |tail -5 |awk -F"to " '{print $2}'|sort|uniq)
	
	for device in ${list}
	do
		count=$(ls -l /dev/${device} | wc -l)
		allclear=$(expr ${allclear} + ${count})
	done
	
	if [ ${allclear} -eq 4 ]
	then
		echo "$(date '+%d/%m %H:%M:%S')  Quectel hardware found." | tee -a "${APPLOG}"
	else
		echo "$(date '+%d/%m %H:%M:%S')  Exit !!,Quectel hardware not found." | tee -a "${APPLOG}"
		exit 0
	fi

}

function start_tshark {
	killall tshark > /dev/null 2>&1
	"${TSHARKBIN}" -i lo -f 'port 4729' -b duration:60 -w "${TMPDIR}/tshark_files" > /dev/null 2>&1 &
	echo $! 
	sleep 3

}

function init_diag {

	killall diag_parser > /dev/null 2>&1
	"${DIAGPBIN}" -g 127.0.0.1 -i /dev/ttyUSB0 > /dev/null 2>&1 &
	echo $! 
	sleep 3
	
}

function init_gps {

	echo -e "AT\r\n" > /dev/ttyUSB2
	sleep 1
	echo -e "AT+QGPS=1\r\n" > /dev/ttyUSB2
	sleep 1
	echo -e "AT+QGPSCFG=\"autogps\",1\r\n" > /dev/ttyUSB2
	sleep 1
	echo -e "AT+QGPSCFG=\"glonassenable\",1\r\n" > /dev/ttyUSB2
	sleep 1
}

function get_coordinates {
	
	if [ "${GPSMODE}" != "FALSE" ]
	then
		if [ "${GPSMODE}" == "quectel" ]
		then
			stty -F /dev/ttyUSB3 9600 cs8 -ignpar -cstopb -icrnl -inlcr -ocrnl -onlcr -echo
			cat /dev/ttyUSB3 > ${TMPDIR}/quectel.gps 2>&1 &
			echo -n -e "AT+QGPSLOC=2\r\n" > /dev/ttyUSB3
			sleep 2
			disown
			kill -9 $! > /dev/null 2>&1
			currentcoord=$(cat ${TMPDIR}/quectel.gps | grep "+QGPSLOC:" |tail -1 | cut -d"," -f2,3)
			rm ${TMPDIR}/quectel.gps
						
			[ "${currentcoord}" == "" ] && currentcoord="0.0, 0.0"
			[[ "${currentcoord}" == *"AT"* ]] && currentcoord="0.0, 0.0"
			[[ "${currentcoord}" == *"Binary file"* ]] && currentcoord="0.0, 0.0"

		fi
		
	else
		currentcoord="0.0 0.0"
	fi

	echo "${currentcoord}"
}

function connect_quectel {
	
	qmicli -d "${CDCDEV}" --dms-uim-get-state | tee /tmp/state
	if [ "$(grep "State:" /tmp/state | cut -d":" -f2)" == " 'initialization-completed'" ]
	then
		rm /tmp/state
		case "${PREFERRED}" in
				"lte")
					qmicli -d "${CDCDEV}" --nas-set-system-selection-preference=lte | tee /tmp/preference
					sleep 2
					[ "$(grep "System selection preference set successfully;" /tmp/preference | cut -d";" -f2)" == " replug your device." ] || echo "$(date '+%d/%m %H:%M:%S') Couldn't establish your prefered network; $(grep ${CDCDEV} /tmp/preference)" | tee -a "${APPLOG}"
					rm /tmp/preference
					qmicli -d "${CDCDEV}" --wds-start-network="${APNSTRING}" --client-no-release-cid > /tmp/network 2>&1
					if [ "$(grep "Network started" /tmp/network  | cut -d"]" -f2 | sed 's/ //')" != "Network started" ]
					then
						qmi-network "${CDCDEV}" start > /tmp/network 2>&1
						grep "Network started successfully" /tmp/network
						[ $? -eq 1 ] && echo "$(date '+%d/%m %H:%M:%S')  Warning, network can't be started !" | tee -a "${APPLOG}"
					fi
					rm /tmp/network
					ifconfig "${QUECTELDEV}" down
					dhclient -r "${QUECTELDEV}"
					ifconfig "${QUECTELDEV}" up
					#######################################################################
					#	Some operators do not support DHCP, please adjust IP address
					#	or require PPP conection setup, modify the line below adding
					#	your connection settings or use a command line connection manager
					dhclient "${QUECTELDEV}"
					#######################################################################
					;;
				"umts")
					qmicli -d "${CDCDEV}" --nas-set-system-selection-preference=umts | tee /tmp/preference
					sleep 2
					[ "$(grep "System selection preference set successfully;" /tmp/preference | cut -d";" -f2)" == " replug your device." ] || echo "$(date '+%d/%m %H:%M:%S') Couldn't establish your prefered network; $(grep ${CDCDEV} /tmp/preference)"  | tee -a "${APPLOG}"
					rm /tmp/preference
					;;
				"gsm")
					qmicli -d "${CDCDEV}" --nas-set-system-selection-preference=gsm | tee /tmp/preference
					sleep 2
					[ "$(grep "System selection preference set successfully;" /tmp/preference | cut -d";" -f2)" == " replug your device." ] || echo "$(date '+%d/%m %H:%M:%S') Couldn't establish your prefered network; $(grep ${CDCDEV} /tmp/preference)"  | tee -a "${APPLOG}"
					rm /tmp/preference
					;;
				"FALSE")
					qmicli -d "${CDCDEV}" --wds-start-network="${APNSTRING}" --client-no-release-cid > /tmp/network 2>&1
					if [ "$(grep "Network started" /tmp/network  | cut -d"]" -f2 | sed 's/ //')" != "Network started" ]
					then
						qmi-network "${CDCDEV}" start > /tmp/network 2>&1
						grep "Network started successfully" /tmp/network
						[ $? -eq 1 ] && echo "$(date '+%d/%m %H:%M:%S')  Warning, network can't be started. If you are re-running the script for a second time, be sure to replug your Quectel device or stop the previous network before run again CellAnalysis." | tee -a "${APPLOG}"
					fi
					rm /tmp/network
					ifconfig "${QUECTELDEV}" down
					dhclient -r "${QUECTELDEV}"
					ifconfig "${QUECTELDEV}" up
					#######################################################################
					#	Some operators do not support DHCP, please adjust IP address
					#	or require PPP conection setup, add below your connection manager
					dhclient "${QUECTELDEV}"
					#######################################################################
					;;
		esac
	else
		echo "$(date '+%d/%m %H:%M:%S') Error, quectel modem is not initialized, check if PIN is disabled and SIM is present."  | tee -a "${APPLOG}"
		rm /tmp/state
		exit 0
	fi
}

function check_files {
	
	[ -n "$(find ${TMPDIR} -maxdepth 0 -type d -empty 2>/dev/null)" ] && NUM=0 || NUM=$(ls -lrt ${TMPDIR}/tshark_files_* | wc -l)
	
	if [ ${NUM} -gt 0 ]
	then
		echo "TRUE"
	else
		echo "FALSE"
	fi

}

function check_num_lin {
	
	INFILE="${1}"
	NUMLIN=$(cat ${INFILE} | wc -l)
	
	echo ${NUMLIN}
}

function check_process {
	
	if [ $# -eq 1 ]
	then
		PROCESSNAME="${1}"
		NPID=$(ps -ef| grep -v "grep" | grep ${PROCESSNAME} | wc -l)
		echo ${NPID}
	fi
}

function check_health_processess {
	
	if [ $(check_process "diag_parser") -eq 0 ]
	then
		DPID=$(init_diag)
		echo "$(date '+%d/%m %H:%M:%S')  diag-parser was not running, new PID: ${DPID}" | tee -a "${APPLOG}"
	fi
	
	if [ $(check_process "tshark") -eq 0 ]
	then
		TPID=$(start_tshark)
		echo "$(date '+%d/%m %H:%M:%S')  thsark was not running, new PID: ${TPID}" | tee -a "${APPLOG}"
	fi
}

function get_current_cell {

	TIMESTAMP=$(date '+%H%M%S')
	DATESTAMP=$(date '+%d/%m')
	coordinates=$(get_coordinates)
	regstatus=$(qmicli -d "${CDCDEV}" --nas-get-serving-system | grep "Registration state:" | cut -d":" -f2)
	qmicli -d "${CDCDEV}" --nas-get-cell-location-info | gawk -v coor="${coordinates}" -v registered="${regstatus}" -v date="${DATESTAMP}" -v timestamp="${TIMESTAMP}" -f ${SCRIPTDIR}/allcells.awk >> ${DEBUGLOG}

}

function analyze_cells {
	
	ls -lrt ${TMPDIR}/tshark_files_* | awk '{print $9}' | while read input_file
	do
		tbdelete=1
		coordinates=$(get_coordinates)
		for test in check_3g_lurej check_4g_turej
		do
			out=$(eval ${test} ${input_file})
			if [ "${out}" != "" ]
			then
				#echo "$(date '+%d/%m %H%M%S')  (\"${coordinates}\")	[Critical] ${out}, more info: ${input_file}" | tee -a "${APPLOG}"
				echo "{\"timestamp\":$(date '+%d/%m %H%M%S'), \"RAT\":'alarm', \"registered\": NA, \"coordinates\":${coordinates}, \"text\":'[Critical] ${out}, more info: ${input_file}'}" | tee -a ${DEBUGLOG}
				tbdelete=0
			fi
		done
			
		for i in 4 3
		do
			test=$( echo "check_${i}g_attach")
			out=$(eval ${test} ${input_file} | cut -d";" -f1)
			if [ ${out} -ne 0 ]
			then
				tbdelete=0
				case ${test} in
					check_4g_attach)
						#echo "$(date '+%d/%m %H:%M:%S')  (\"${coordinates}\")  [Critical] 4G Attach using IMSI ${out} times, more info: ${input_file}" | tee -a "${APPLOG}"
						echo "{\"timestamp\":$(date '+%d/%m %H%M%S'), \"RAT\":'alarm', \"registered\": NA, \"coordinates\":${coordinates}, \"text\":'[Critical] 4G Attach using IMSI ${out} times, more info: ${input_file}'}" | tee -a ${DEBUGLOG}
						;;
					check_3g_attach)
						text=$(check_3g_attach ${input_file}| cut -d";" -f2)
						#echo "$(date '+%d/%m %H:%M:%S')  (\"${coordinates}\")  [Critical] ${text} GPRS Attach using IMSI ${out} times, more info: ${input_file}" | tee -a "${APPLOG}"
						echo "{\"timestamp\":$(date '+%d/%m %H%M%S'), \"RAT\":'alarm', \"registered\": NA, \"coordinates\":${coordinates}, \"text\":'[Critical] ${text} GPRS Attach using IMSI ${out} times, more info: ${input_file}'}" | tee -a ${DEBUGLOG}
						;;
				esac
			fi
			
		done
		
		if [ ${tbdelete} -eq 1 ]
		then
			rm ${input_file}
		else
			mv ${input_file} ${OUTDIR}
		fi
		
	done
} 

function check_3g_lurej {

	filter="gsm_a.dtap.msg_mm_type == 0x04"
	"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "${filter}" > /tmp/3g_lurej 2>&1
	grep "Location Updating Reject" /tmp/3g_lurej > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		arfcn=$(grep "GSM TAP Header, ARFCN:" /tmp/3g_lurej |cut -d":" -f2 | cut -d"," -f1)
		[ "${arfcn}" == " 0 (Downlink)" ] && tech="3G" || tech=$(echo "2G, ${arfcn}" | sed 's/(Downlink)//')
		lurej_cause="${tech} Location Updating Reject: $(grep "Reject cause:" /tmp/3g_lurej | cut -d":" -f2 | cut -d"\"" -f1)"
	else
		lurej_cause=""
	fi
	rm /tmp/3g_lurej

	echo "${lurej_cause}"
}

function check_4g_turej {

	filter="nas_eps.nas_msg_emm_type == 0x4b"
	"${TSHARKBIN}" -r $1 -c 1 -T pdml -2 -R "${filter}" > /tmp/4g_turej 2>&1
	grep "Tracking area update reject" /tmp/4g_turej > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		turej_cause="4G Tracking area update reject: $(grep "Cause:" /tmp/4g_turej | cut -d":" -f2 | cut -d"\"" -f1)"
	else
		turej_cause=""
	fi
	rm /tmp/4g_turej

	echo "${turej_cause}"
}

function check_4g_attach {

	filter="nas_eps.nas_msg_emm_type == 0x41"
	"${TSHARKBIN}" -r $1 -T pdml -2 -R "${filter}" > /tmp/4g_attach 2>&1
	grep "NAS EPS Mobility Management Message Type: Attach request (0x41)" /tmp/4g_attach > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		bad4gcases=$(grep -v "EPS attach type: Combined EPS/IMSI attach" /tmp/4g_attach | grep -c "Type of identity: IMSI")
	else
		bad4gcases=0
	fi
	rm /tmp/4g_attach

	echo "${bad4gcases}"
}

function check_3g_attach {

	filter="gsm_a.dtap.msg_gmm_type == 0x01"
	"${TSHARKBIN}" -r $1 -T pdml -2 -R "${filter}" > /tmp/3g_attach 2>&1
	grep "GPRS Mobility Management Message Type: Attach Request" /tmp/3g_attach > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		arfcn=$(grep "GSM TAP Header, ARFCN:" /tmp/3g_attach |cut -d":" -f2 | cut -d"," -f1| sort | uniq)
		[ "${arfcn}" == " 0 (Uplink)" ] && tech="3G" || tech=$(echo "2G, ${arfcn}" | sed 's/(Uplink)//')
		bad3gcases=$(grep -c "Mobile Identity Type: IMSI" /tmp/3g_attach)
	else
		arfcn="NA"
		tech="NA"
		bad3gcases=0
	fi
	rm /tmp/3g_attach

	echo "${bad3gcases};${tech}"
}
###########################################################################################################
#	Main program
###########################################################################################################

disclaimer

if [ $# -eq 0 ]
then
	usage	
	exit 0
else
	while getopts "hm:g:p:v" opt; do
	  case $opt in
	    m)
	      MODE="${OPTARG}"
	      case "${MODE}" in
			"quectel");;
			*)
			 echo "$(date '+%d/%m %H:%M:%S')  Bad parameter, "${OPTARG}""
			 usage
			 exit 1
	      esac
	      echo "$(date '+%d/%m %H:%M:%S')  Using mode: $OPTARG" | tee -a "${APPLOG}"
	      ;;
	     p)
	      PREFERRED="${OPTARG}"
			case "${PREFERRED}" in
				"lte");;
				"umts");;
				"gsm");;
				*)
				echo "$(date '+%d/%m %H:%M:%S')  Bad parameter, "${OPTARG}", -p option can only be used with: lte, umts or gsm." | tee -a "${APPLOG}"
				usage
				exit 1
			esac
	      echo "$(date '+%d/%m %H:%M:%S')  Using preferred network: ${PREFERRED}" | tee -a "${APPLOG}"
	      echo ">>>>>>>>>>>>>>>>>>>>>>>>	IMPORTANT; You have to replug your device every time the preferred network changes." | tee -a "${APPLOG}"
	      ;;
	     g)
	      GPSMODE="${OPTARG}"
	      case "${GPSMODE}" in
		 "variable");;
		 "quectel");;
			*)
			echo "$(date '+%d/%m %H:%M:%S')  Bad parameter, "${OPTARG}", -g option can only be used with: quectel or variable values." | tee -a "${APPLOG}"
			usage
			exit 1
		 esac
	      echo "$(date '+%d/%m %H:%M:%S')  Using GPS: ${GPSMODE}" | tee -a "${APPLOG}"
	      ;;
	    v)
	      CVERBOSE="TRUE"
	      ;;
	    \?)
	      echo "$(date '+%d/%m %H:%M:%S')  Invalid option: -$OPTARG" >&2 | tee -a "${APPLOG}"
	      usage
	      exit 1
	      ;;
	    h)
	      usage
	      exit 0
	      ;;
	    *)
	      echo "$(date '+%d/%m %H:%M:%S')  Option -$OPTARG requires an argument." >&2 | tee -a "${APPLOG}"
	      exit 1
	      ;;
	  esac
	done
fi

###########################################################################################################
#
#	Main loop
#
###########################################################################################################

check_bin

check_hw

TPID=$(start_tshark)
echo "$(date '+%d/%m %H:%M:%S')  tshark running: ${TPID}." | tee -a "${APPLOG}"
	
DPID=$(init_diag)
echo "$(date '+%d/%m %H:%M:%S')  diag-parser running: ${DPID}." | tee -a "${APPLOG}"

connect_quectel
echo "$(date '+%d/%m %H:%M:%S')  quectel modem initialized." | tee -a "${APPLOG}"

echo "$(date '+%d/%m %H:%M:%S')  enetering main loop, cancel at any time by presing Ctrl+C" | tee -a "${APPLOG}"

if [ ${GPSMODE} == "gpsd" ]
then
	init_gps
	sleep 1
fi

while true
do	
	
	get_current_cell
	
	READY=$(check_files)
	
	if [ ${READY} == "TRUE" ]
	then
		analyze_cells "${MODE}"
		echo "$(date '+%d/%m %H:%M:%S')  analyzing cells..." | tee -a "${APPLOG}"
	fi
	
	check_health_processess

	sleep 5
	done
fi
